# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..core.unchecked_base_model import construct_type
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.create_datapoint_request import CreateDatapointRequest
from ..types.datapoint_response import DatapointResponse
from ..types.dataset_response import DatasetResponse
from ..types.http_validation_error import HttpValidationError
from ..types.list_datasets import ListDatasets
from ..types.paginated_datapoint_response import PaginatedDatapointResponse
from ..types.paginated_dataset_response import PaginatedDatasetResponse
from ..types.project_sort_by import ProjectSortBy
from ..types.sort_order import SortOrder
from ..types.update_dateset_action import UpdateDatesetAction
from ..types.version_status import VersionStatus

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DatasetsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        page: typing.Optional[int] = None,
        size: typing.Optional[int] = None,
        name: typing.Optional[str] = None,
        user_filter: typing.Optional[str] = None,
        sort_by: typing.Optional[ProjectSortBy] = None,
        order: typing.Optional[SortOrder] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[DatasetResponse]:
        """
        Get a list of Datasets.

        Parameters
        ----------
        page : typing.Optional[int]
            Page offset for pagination.

        size : typing.Optional[int]
            Page size for pagination. Number of Datasets to fetch.

        name : typing.Optional[str]
            Case-insensitive filter for Dataset name.

        user_filter : typing.Optional[str]
            Case-insensitive filter for users in the Dataset. This filter matches against both email address and name of users.

        sort_by : typing.Optional[ProjectSortBy]
            Field to sort Datasets by

        order : typing.Optional[SortOrder]
            Direction to sort by.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[DatasetResponse]
            Successful Response

        Examples
        --------
        from humanloop.client import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        response = client.datasets.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        page = page or 1
        _response = self._client_wrapper.httpx_client.request(
            "datasets",
            method="GET",
            params={
                "page": page,
                "size": size,
                "name": name,
                "user_filter": user_filter,
                "sort_by": sort_by,
                "order": order,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(PaginatedDatasetResponse, construct_type(type_=PaginatedDatasetResponse, object_=_response.json()))  # type: ignore
                _has_next = True
                _get_next = lambda: self.list(
                    page=page + 1,
                    size=size,
                    name=name,
                    user_filter=user_filter,
                    sort_by=sort_by,
                    order=order,
                    request_options=request_options,
                )
                _items = _parsed_response.records
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        datapoints: typing.Sequence[CreateDatapointRequest],
        version_id: typing.Optional[str] = None,
        environment: typing.Optional[str] = None,
        path: typing.Optional[str] = OMIT,
        id: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        commit_message: typing.Optional[str] = OMIT,
        action: typing.Optional[UpdateDatesetAction] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetResponse:
        """
        Create a Dataset or update it with a new version if it already exists.

        Datasets are identified by the `ID` or their `path`. The datapoints determine the versions of the Dataset.

        By default, the new Dataset version will be set to the list of Datapoints provided in
        the request. You can also create a new version by adding or removing Datapoints from an existing version
        by specifying `action` as `add` or `remove` respectively. In this case, you may specify
        the `version_id` or `environment` query parameters to identify the existing version to base
        the new version on. If neither is provided, the default deployed version will be used.

        If you provide a commit message, then the new version will be committed;
        otherwise it will be uncommitted. If you try to commit an already committed version,
        an exception will be raised.

        Humanloop also deduplicates Datapoints. If you try to add a Datapoint that already
        exists, it will be ignored. If you intentionally want to add a duplicate Datapoint,
        you can add a unique identifier to the Datapoint's inputs such as `{_dedupe_id: <unique ID>}`.

        Parameters
        ----------
        datapoints : typing.Sequence[CreateDatapointRequest]
            The Datapoints to create this Dataset version with. Modify the `action` field to determine how these Datapoints are used.

        version_id : typing.Optional[str]
            ID of the specific Dataset version to base the created Version on. Only used when `action` is `"add"` or `"remove"`.

        environment : typing.Optional[str]
            An environment tag to identify a deployed Version to base the created Version on. Only used when `action` is `"add"` or `"remove"`.

        path : typing.Optional[str]
            Path of the Dataset including the Dataset name, which is used as a unique identifier.

        id : typing.Optional[str]
            ID for an existing Dataset to update.

        name : typing.Optional[str]
            Name of the Dataset, which is used as a unique identifier.

        commit_message : typing.Optional[str]
            Message describing the changes made. If provided, a committed version of the Dataset is created. Otherwise, an uncommitted version is created.

        action : typing.Optional[UpdateDatesetAction]
            The action to take with the provided Datapoints.

             - If `"set"`, the created version will only contain the Datapoints provided in this request.
             - If `"add"`, the created version will contain the Datapoints provided in this request in addition to the Datapoints in the target version.
             - If `"remove"`, the created version will contain the Datapoints in the target version except for the Datapoints provided in this request.

            If `"add"` or `"remove"`, one of the `version_id` or `environment` query parameters may be provided.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetResponse
            Successful Response

        Examples
        --------
        from humanloop import CreateDatapointRequest
        from humanloop.client import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        client.datasets.create(
            datapoints=[CreateDatapointRequest()],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "datasets",
            method="POST",
            params={"version_id": version_id, "environment": environment},
            json={
                "path": path,
                "id": id,
                "name": name,
                "commit_message": commit_message,
                "datapoints": datapoints,
                "action": action,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(DatasetResponse, construct_type(type_=DatasetResponse, object_=_response.json()))  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self,
        id: str,
        *,
        version_id: typing.Optional[str] = None,
        environment: typing.Optional[str] = None,
        include_datapoints: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetResponse:
        """
        Retrieve the Dataset with the given ID.

        Unless `include_datapoints` is set to `true`, the response will not include
        the Datapoints.
        Use the List Datapoints endpoint (`GET /{id}/datapoints`) to efficiently
        retrieve Datapoints for a large Dataset.

        By default the deployed version of the Dataset is returned. Use the query parameters
        `version_id` or `environment` to target a specific version of the Dataset.

        Parameters
        ----------
        id : str
            Unique identifier for Dataset.

        version_id : typing.Optional[str]
            A specific Version ID of the Dataset to retrieve.

        environment : typing.Optional[str]
            An environment tag to retrieve a deployed Version from.

        include_datapoints : typing.Optional[bool]
            If set to `true`, include all Datapoints in the response. Defaults to `false`. Consider using the paginated List Datapoints endpoint instead.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetResponse
            Successful Response

        Examples
        --------
        from humanloop.client import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        client.datasets.get(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"datasets/{jsonable_encoder(id)}",
            method="GET",
            params={"version_id": version_id, "environment": environment, "include_datapoints": include_datapoints},
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(DatasetResponse, construct_type(type_=DatasetResponse, object_=_response.json()))  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete the Dataset with the given ID.

        Parameters
        ----------
        id : str
            Unique identifier for Dataset.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from humanloop.client import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        client.datasets.delete(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"datasets/{jsonable_encoder(id)}", method="DELETE", request_options=request_options
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        id: str,
        *,
        path: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetResponse:
        """
        Update the Dataset with the given ID.

        Parameters
        ----------
        id : str
            Unique identifier for Dataset.

        path : typing.Optional[str]
            Path of the Dataset including the Dataset name, which is used as a unique identifier.

        name : typing.Optional[str]
            Name of the Dataset, which is used as a unique identifier.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetResponse
            Successful Response

        Examples
        --------
        from humanloop.client import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        client.datasets.update(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"datasets/{jsonable_encoder(id)}",
            method="PATCH",
            json={"path": path, "name": name},
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(DatasetResponse, construct_type(type_=DatasetResponse, object_=_response.json()))  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def list_datapoints(
        self,
        id: str,
        *,
        version_id: typing.Optional[str] = None,
        environment: typing.Optional[str] = None,
        page: typing.Optional[int] = None,
        size: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[DatapointResponse]:
        """
        List all Datapoints for the Dataset with the given ID.

        Parameters
        ----------
        id : str
            Unique identifier for Dataset.

        version_id : typing.Optional[str]
            A specific Version ID of the Dataset to retrieve.

        environment : typing.Optional[str]
            An environment tag to retrieve a deployed Version from.

        page : typing.Optional[int]
            Page number for pagination.

        size : typing.Optional[int]
            Page size for pagination. Number of Datapoints to fetch.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[DatapointResponse]
            Successful Response

        Examples
        --------
        from humanloop.client import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        response = client.datasets.list_datapoints(
            id="id",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        page = page or 1
        _response = self._client_wrapper.httpx_client.request(
            f"datasets/{jsonable_encoder(id)}/datapoints",
            method="GET",
            params={"version_id": version_id, "environment": environment, "page": page, "size": size},
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(PaginatedDatapointResponse, construct_type(type_=PaginatedDatapointResponse, object_=_response.json()))  # type: ignore
                _has_next = True
                _get_next = lambda: self.list_datapoints(
                    id,
                    version_id=version_id,
                    environment=environment,
                    page=page + 1,
                    size=size,
                    request_options=request_options,
                )
                _items = _parsed_response.records
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def list_versions(
        self,
        id: str,
        *,
        status: typing.Optional[VersionStatus] = None,
        environment: typing.Optional[str] = None,
        evaluation_aggregates: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListDatasets:
        """
        Get a list of the versions for a Dataset.

        Parameters
        ----------
        id : str
            Unique identifier for Dataset.

        status : typing.Optional[VersionStatus]
            Filter versions by status: 'uncommitted', 'committed'. If no status is provided, all versions are returned.

        environment : typing.Optional[str]
            Filter versions by environment tag. If no environment is provided, all versions are returned.

        evaluation_aggregates : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListDatasets
            Successful Response

        Examples
        --------
        from humanloop.client import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        client.datasets.list_versions(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"datasets/{jsonable_encoder(id)}/versions",
            method="GET",
            params={"status": status, "environment": environment, "evaluation_aggregates": evaluation_aggregates},
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(ListDatasets, construct_type(type_=ListDatasets, object_=_response.json()))  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def commit(
        self, id: str, version_id: str, *, commit_message: str, request_options: typing.Optional[RequestOptions] = None
    ) -> DatasetResponse:
        """
        Commit the Dataset Version with the given ID.

        Parameters
        ----------
        id : str
            Unique identifier for Dataset.

        version_id : str
            Unique identifier for the specific version of the Dataset.

        commit_message : str
            Message describing the changes made.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetResponse
            Successful Response

        Examples
        --------
        from humanloop.client import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        client.datasets.commit(
            id="id",
            version_id="version_id",
            commit_message="commit_message",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"datasets/{jsonable_encoder(id)}/versions/{jsonable_encoder(version_id)}/commit",
            method="POST",
            json={"commit_message": commit_message},
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(DatasetResponse, construct_type(type_=DatasetResponse, object_=_response.json()))  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def deploy(
        self, id: str, version_id: str, *, environment_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> DatasetResponse:
        """
        Deploy Dataset to Environment.

        Set the deployed Version for the specified Environment.

        Parameters
        ----------
        id : str
            Unique identifier for Dataset.

        version_id : str
            Unique identifier for the specific version of the Dataset.

        environment_id : str
            Unique identifier for the Environment to deploy the Version to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetResponse
            Successful Response

        Examples
        --------
        from humanloop.client import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        client.datasets.deploy(
            id="id",
            version_id="version_id",
            environment_id="environment_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"datasets/{jsonable_encoder(id)}/versions/{jsonable_encoder(version_id)}/deploy",
            method="POST",
            params={"environment_id": environment_id},
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(DatasetResponse, construct_type(type_=DatasetResponse, object_=_response.json()))  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncDatasetsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        page: typing.Optional[int] = None,
        size: typing.Optional[int] = None,
        name: typing.Optional[str] = None,
        user_filter: typing.Optional[str] = None,
        sort_by: typing.Optional[ProjectSortBy] = None,
        order: typing.Optional[SortOrder] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[DatasetResponse]:
        """
        Get a list of Datasets.

        Parameters
        ----------
        page : typing.Optional[int]
            Page offset for pagination.

        size : typing.Optional[int]
            Page size for pagination. Number of Datasets to fetch.

        name : typing.Optional[str]
            Case-insensitive filter for Dataset name.

        user_filter : typing.Optional[str]
            Case-insensitive filter for users in the Dataset. This filter matches against both email address and name of users.

        sort_by : typing.Optional[ProjectSortBy]
            Field to sort Datasets by

        order : typing.Optional[SortOrder]
            Direction to sort by.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[DatasetResponse]
            Successful Response

        Examples
        --------
        from humanloop.client import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )
        response = await client.datasets.list()
        async for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        async for page in response.iter_pages():
            yield page
        """
        page = page or 1
        _response = await self._client_wrapper.httpx_client.request(
            "datasets",
            method="GET",
            params={
                "page": page,
                "size": size,
                "name": name,
                "user_filter": user_filter,
                "sort_by": sort_by,
                "order": order,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(PaginatedDatasetResponse, construct_type(type_=PaginatedDatasetResponse, object_=_response.json()))  # type: ignore
                _has_next = True
                _get_next = lambda: self.list(
                    page=page + 1,
                    size=size,
                    name=name,
                    user_filter=user_filter,
                    sort_by=sort_by,
                    order=order,
                    request_options=request_options,
                )
                _items = _parsed_response.records
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        datapoints: typing.Sequence[CreateDatapointRequest],
        version_id: typing.Optional[str] = None,
        environment: typing.Optional[str] = None,
        path: typing.Optional[str] = OMIT,
        id: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        commit_message: typing.Optional[str] = OMIT,
        action: typing.Optional[UpdateDatesetAction] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetResponse:
        """
        Create a Dataset or update it with a new version if it already exists.

        Datasets are identified by the `ID` or their `path`. The datapoints determine the versions of the Dataset.

        By default, the new Dataset version will be set to the list of Datapoints provided in
        the request. You can also create a new version by adding or removing Datapoints from an existing version
        by specifying `action` as `add` or `remove` respectively. In this case, you may specify
        the `version_id` or `environment` query parameters to identify the existing version to base
        the new version on. If neither is provided, the default deployed version will be used.

        If you provide a commit message, then the new version will be committed;
        otherwise it will be uncommitted. If you try to commit an already committed version,
        an exception will be raised.

        Humanloop also deduplicates Datapoints. If you try to add a Datapoint that already
        exists, it will be ignored. If you intentionally want to add a duplicate Datapoint,
        you can add a unique identifier to the Datapoint's inputs such as `{_dedupe_id: <unique ID>}`.

        Parameters
        ----------
        datapoints : typing.Sequence[CreateDatapointRequest]
            The Datapoints to create this Dataset version with. Modify the `action` field to determine how these Datapoints are used.

        version_id : typing.Optional[str]
            ID of the specific Dataset version to base the created Version on. Only used when `action` is `"add"` or `"remove"`.

        environment : typing.Optional[str]
            An environment tag to identify a deployed Version to base the created Version on. Only used when `action` is `"add"` or `"remove"`.

        path : typing.Optional[str]
            Path of the Dataset including the Dataset name, which is used as a unique identifier.

        id : typing.Optional[str]
            ID for an existing Dataset to update.

        name : typing.Optional[str]
            Name of the Dataset, which is used as a unique identifier.

        commit_message : typing.Optional[str]
            Message describing the changes made. If provided, a committed version of the Dataset is created. Otherwise, an uncommitted version is created.

        action : typing.Optional[UpdateDatesetAction]
            The action to take with the provided Datapoints.

             - If `"set"`, the created version will only contain the Datapoints provided in this request.
             - If `"add"`, the created version will contain the Datapoints provided in this request in addition to the Datapoints in the target version.
             - If `"remove"`, the created version will contain the Datapoints in the target version except for the Datapoints provided in this request.

            If `"add"` or `"remove"`, one of the `version_id` or `environment` query parameters may be provided.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetResponse
            Successful Response

        Examples
        --------
        from humanloop import CreateDatapointRequest
        from humanloop.client import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )
        await client.datasets.create(
            datapoints=[CreateDatapointRequest()],
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "datasets",
            method="POST",
            params={"version_id": version_id, "environment": environment},
            json={
                "path": path,
                "id": id,
                "name": name,
                "commit_message": commit_message,
                "datapoints": datapoints,
                "action": action,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(DatasetResponse, construct_type(type_=DatasetResponse, object_=_response.json()))  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self,
        id: str,
        *,
        version_id: typing.Optional[str] = None,
        environment: typing.Optional[str] = None,
        include_datapoints: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetResponse:
        """
        Retrieve the Dataset with the given ID.

        Unless `include_datapoints` is set to `true`, the response will not include
        the Datapoints.
        Use the List Datapoints endpoint (`GET /{id}/datapoints`) to efficiently
        retrieve Datapoints for a large Dataset.

        By default the deployed version of the Dataset is returned. Use the query parameters
        `version_id` or `environment` to target a specific version of the Dataset.

        Parameters
        ----------
        id : str
            Unique identifier for Dataset.

        version_id : typing.Optional[str]
            A specific Version ID of the Dataset to retrieve.

        environment : typing.Optional[str]
            An environment tag to retrieve a deployed Version from.

        include_datapoints : typing.Optional[bool]
            If set to `true`, include all Datapoints in the response. Defaults to `false`. Consider using the paginated List Datapoints endpoint instead.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetResponse
            Successful Response

        Examples
        --------
        from humanloop.client import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )
        await client.datasets.get(
            id="id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"datasets/{jsonable_encoder(id)}",
            method="GET",
            params={"version_id": version_id, "environment": environment, "include_datapoints": include_datapoints},
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(DatasetResponse, construct_type(type_=DatasetResponse, object_=_response.json()))  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete the Dataset with the given ID.

        Parameters
        ----------
        id : str
            Unique identifier for Dataset.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from humanloop.client import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )
        await client.datasets.delete(
            id="id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"datasets/{jsonable_encoder(id)}", method="DELETE", request_options=request_options
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        id: str,
        *,
        path: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DatasetResponse:
        """
        Update the Dataset with the given ID.

        Parameters
        ----------
        id : str
            Unique identifier for Dataset.

        path : typing.Optional[str]
            Path of the Dataset including the Dataset name, which is used as a unique identifier.

        name : typing.Optional[str]
            Name of the Dataset, which is used as a unique identifier.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetResponse
            Successful Response

        Examples
        --------
        from humanloop.client import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )
        await client.datasets.update(
            id="id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"datasets/{jsonable_encoder(id)}",
            method="PATCH",
            json={"path": path, "name": name},
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(DatasetResponse, construct_type(type_=DatasetResponse, object_=_response.json()))  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def list_datapoints(
        self,
        id: str,
        *,
        version_id: typing.Optional[str] = None,
        environment: typing.Optional[str] = None,
        page: typing.Optional[int] = None,
        size: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[DatapointResponse]:
        """
        List all Datapoints for the Dataset with the given ID.

        Parameters
        ----------
        id : str
            Unique identifier for Dataset.

        version_id : typing.Optional[str]
            A specific Version ID of the Dataset to retrieve.

        environment : typing.Optional[str]
            An environment tag to retrieve a deployed Version from.

        page : typing.Optional[int]
            Page number for pagination.

        size : typing.Optional[int]
            Page size for pagination. Number of Datapoints to fetch.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[DatapointResponse]
            Successful Response

        Examples
        --------
        from humanloop.client import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )
        response = await client.datasets.list_datapoints(
            id="id",
        )
        async for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        async for page in response.iter_pages():
            yield page
        """
        page = page or 1
        _response = await self._client_wrapper.httpx_client.request(
            f"datasets/{jsonable_encoder(id)}/datapoints",
            method="GET",
            params={"version_id": version_id, "environment": environment, "page": page, "size": size},
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(PaginatedDatapointResponse, construct_type(type_=PaginatedDatapointResponse, object_=_response.json()))  # type: ignore
                _has_next = True
                _get_next = lambda: self.list_datapoints(
                    id,
                    version_id=version_id,
                    environment=environment,
                    page=page + 1,
                    size=size,
                    request_options=request_options,
                )
                _items = _parsed_response.records
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def list_versions(
        self,
        id: str,
        *,
        status: typing.Optional[VersionStatus] = None,
        environment: typing.Optional[str] = None,
        evaluation_aggregates: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListDatasets:
        """
        Get a list of the versions for a Dataset.

        Parameters
        ----------
        id : str
            Unique identifier for Dataset.

        status : typing.Optional[VersionStatus]
            Filter versions by status: 'uncommitted', 'committed'. If no status is provided, all versions are returned.

        environment : typing.Optional[str]
            Filter versions by environment tag. If no environment is provided, all versions are returned.

        evaluation_aggregates : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListDatasets
            Successful Response

        Examples
        --------
        from humanloop.client import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )
        await client.datasets.list_versions(
            id="id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"datasets/{jsonable_encoder(id)}/versions",
            method="GET",
            params={"status": status, "environment": environment, "evaluation_aggregates": evaluation_aggregates},
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(ListDatasets, construct_type(type_=ListDatasets, object_=_response.json()))  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def commit(
        self, id: str, version_id: str, *, commit_message: str, request_options: typing.Optional[RequestOptions] = None
    ) -> DatasetResponse:
        """
        Commit the Dataset Version with the given ID.

        Parameters
        ----------
        id : str
            Unique identifier for Dataset.

        version_id : str
            Unique identifier for the specific version of the Dataset.

        commit_message : str
            Message describing the changes made.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetResponse
            Successful Response

        Examples
        --------
        from humanloop.client import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )
        await client.datasets.commit(
            id="id",
            version_id="version_id",
            commit_message="commit_message",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"datasets/{jsonable_encoder(id)}/versions/{jsonable_encoder(version_id)}/commit",
            method="POST",
            json={"commit_message": commit_message},
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(DatasetResponse, construct_type(type_=DatasetResponse, object_=_response.json()))  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def deploy(
        self, id: str, version_id: str, *, environment_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> DatasetResponse:
        """
        Deploy Dataset to Environment.

        Set the deployed Version for the specified Environment.

        Parameters
        ----------
        id : str
            Unique identifier for Dataset.

        version_id : str
            Unique identifier for the specific version of the Dataset.

        environment_id : str
            Unique identifier for the Environment to deploy the Version to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DatasetResponse
            Successful Response

        Examples
        --------
        from humanloop.client import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )
        await client.datasets.deploy(
            id="id",
            version_id="version_id",
            environment_id="environment_id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"datasets/{jsonable_encoder(id)}/versions/{jsonable_encoder(version_id)}/deploy",
            method="POST",
            params={"environment_id": environment_id},
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(DatasetResponse, construct_type(type_=DatasetResponse, object_=_response.json()))  # type: ignore
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(HttpValidationError, construct_type(type_=HttpValidationError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
